@@ -27,12 +27,13 @@ Copyright_License {
 #include "Widget/RowFormWidget.hpp"
 #include "Form/DataField/Enum.hpp"
 #include "Form/DataField/GeoPoint.hpp"
+#include "Form/DataField/Listener.hpp"
 #include "UIGlobals.hpp"
 #include "Waypoint/Waypoint.hpp"
 #include "Interface.hpp"
 #include "Language/Language.hpp"
 
-class WaypointEditWidget final : public RowFormWidget {
+class WaypointEditWidget final : public RowFormWidget, DataFieldListener {
   enum Rows {
     NAME,
     COMMENT,
@@ -43,9 +44,11 @@ class WaypointEditWidget final : public RowFormWidget {
 
   Waypoint value;
 
+  bool modified;
+
 public:
   WaypointEditWidget(const DialogLook &look, Waypoint _value)
-    :RowFormWidget(look), value(_value) {}
+    :RowFormWidget(look), value(_value), modified(false) {}
 
   const Waypoint &GetValue() const {
     return value;
@@ -55,6 +58,11 @@ private:
   /* virtual methods from Widget */
   void Prepare(ContainerWindow &parent, const PixelRect &rc) override;
   bool Save(bool &changed) override;
+
+  /* virtual methods from DataFieldListener */
+  void OnModified(gcc_unused DataField &df) override {
+    modified = true;
+  }
 };
 
 static constexpr StaticEnumChoice waypoint_types[] = {
@@ -68,21 +76,25 @@ void
 WaypointEditWidget::Prepare(gcc_unused ContainerWindow &parent,
                             gcc_unused const PixelRect &rc)
 {
-  AddText(_("Name"), nullptr, value.name.c_str());
-  AddText(_("Comment"), nullptr, value.comment.c_str());
-  Add(_("Location"), nullptr, new GeoPointDataField(value.location,UIGlobals::GetFormatSettings().coordinate_format));
+  AddText(_("Name"), nullptr, value.name.c_str(), this);
+  AddText(_("Comment"), nullptr, value.comment.c_str(), this);
+  Add(_("Location"), nullptr,
+      new GeoPointDataField(value.location,
+                            UIGlobals::GetFormatSettings().coordinate_format,
+                            this));
   AddFloat(_("Altitude"), nullptr,
            _T("%.0f %s"), _T("%.0f"),
            fixed(0), fixed(30000), fixed(5), false,
            UnitGroup::ALTITUDE, value.elevation);
   AddEnum(_("Type"), nullptr, waypoint_types,
-          value.IsAirport() ? 1u : (value.IsLandable() ? 2u : 0u ));
+          value.IsAirport() ? 1u : (value.IsLandable() ? 2u : 0u),
+          this);
 }
 
 bool
 WaypointEditWidget::Save(bool &_changed)
 {
-  bool changed = false;
+  bool changed = modified;
   value.name = GetValueString(NAME);
   value.comment = GetValueString(COMMENT);
   value.location = ((GeoPointDataField &)GetDataField(LOCATION)).GetValue();
@@ -128,7 +140,7 @@ dlgWaypointEditShowModal(Waypoint &way_point)
   const int result = dialog.ShowModal();
   dialog.StealWidget();
 
-  if (result != mrOK)
+  if (result != mrOK || !dialog.GetChanged())
     return false;
 
   way_point = widget.GetValue();
